SUBROUTINE ADVECT(FIELDOLD,FIELD,DELFIELD,FAVER,ICODE)
!
!           ADVECT integrates the equation
!
!           d(FIELD*D)/d(T) + d(FIELD*U*D)/d(X) + d(FIELD*V*D)/d(Y) +
!       
!           d(FIELD*W)/d(SIGMA) = d(AH*d(FIELD*D)/d(X))/d(X) +
!                                 d(AH*d(FIELD*D)/d(Y))/d(Y)
!
!           from timestep n to timestep n+1
!           with a superbee limiter TVD scheme.
!
!           d(*) is the differential operator,
!           FIELD is the scalar field to be integrated,
!           D is the dynamic depth,
!           T is time,
!           X and Y are the horisontal coordinates,
!           U is the horisontal velocity in X-direction,
!           V is the horisontal velocity in Y-direction,
!           SIGMA is the vertical coordinate,
!           W is the vertical velocity. 
!           
!           FAVER is the average value of the scalar field which is 
!                 subtracted from FIELD before differences are 
!                 computed to reduce the cancellation errors.
!           ICODE is an integer flag to signal which scalar field
!                 that is advected. ICODE is needed when river-runoff
!                 is included.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
USE MOD_BOUND
IMPLICIT NONE
REAL FIELDOLD(IM,JM,KB),FIELD(IM,JM,KB),DELFIELD(IM,JM,KB),FAVER
INTEGER ICODE
!
!   Local variables.
!
REAL, DIMENSION(IM,JM,KB) :: XFLUX,YFLUX,ZFLUX
REAL, DIMENSION(IM,JM,KB) :: XFLUXD,YFLUXD
REAL, DIMENSION(IM,JM,KB) :: DIFF1,DIFF2,FNEW
!
REAL AREA,NETFLUX,VOLUME,VOLUMEP
REAL AREAT,AREATU,AREATV,AREATW
REAL EPSILON,ONESIX
INTEGER I,J,K
REAL FRAC,FLUX,FIELDRIVER
REAL SUM,PROD
!
EPSILON = 1.E-5
ONESIX = 1.D0/6.D0
!
!   Define FIELD(KB) = FIELD(KB-1)
!
FIELD(1:IM,1:JM,KB) = FIELD(1:IM,1:JM,KB-1)
!
  FIELD = FIELD - FAVER
  FIELDOLD = FIELDOLD - FAVER
  DELFIELD = 0.
!
XFLUX = 0.
XFLUXD = 0.
YFLUX = 0.
YFLUXD = 0.
ZFLUX = 0.
!
!   Compute F*U*D in U-points of the
!   Arakawa C-grid.
!
!   Compute the sum of the advective and diffusive
!   fluxes  in X-direction.
!
IF (RIVLOG) THEN
  DO I = 1,NUMC
    IF (ICODE.EQ.1) THEN
!
!  Salinity of river water
!
      FIELDRIVER = SRIVER(I)
    ELSEIF (ICODE.EQ.2) THEN
!
! Temperature of the river water
!
      FIELDRIVER = TRIVER(I)
    ELSE
      WRITE(*,*)'ERROR IN ADVECT'
      STOP
    ENDIF
    FIELDRIVER = FIELDRIVER - FAVER
    DO K = KRIVER1(I),KRIVER2(I)
      FRAC = DZ(K)/DZRIVER(I)
      FLUX = FRAC*FRIVER(I)*FIELDRIVER 
      IF (CODERIVER(I).EQ.1) THEN
        XFLUX(IRIVER(I),JRIVER(I),K) = FLUX
      ELSEIF (CODERIVER(I).EQ.2) THEN
        YFLUX(IRIVER(I),JRIVER(I),K) = FLUX
      ELSEIF (CODERIVER(I).EQ.3) THEN
        XFLUX(IRIVER(I)+1,JRIVER(I),K) = -FLUX
      ELSEIF (CODERIVER(I).EQ.4) THEN
        YFLUX(IRIVER(I),JRIVER(I)+1,K) = -FLUX
      ELSE
        WRITE(*,*)'ERROR IN ADVECT-2'
        STOP
      ENDIF
    END DO
  END DO
ENDIF
! 
!  DIFF1
!
DIFF1 = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 1,JM
  DO I = 2,IM
    DO K = 1,KB-1
      DIFF1(I,J,K) = (FIELD(I,J,K) -FIELD(I-1,J,K))*DUM(I,J)
    END DO
  END DO
END DO 
!
!  DIFF2
!
DIFF2 = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 1,JM
  DO I = 1,IM-1
    DO K = 1,KB-1
      SUM = DIFF1(I,J,K)+DIFF1(I+1,J,K)
      PROD = DIFF1(I,J,K)*DIFF1(I+1,J,K)
      IF (ABS(SUM).GT.EPSILON .AND. PROD.GT.EPSILON) THEN
        DIFF2(I,J,K) = 2.0*PROD/SUM
        DIFF2(I,J,K) = DIFF2(I,J,K)*FSM(I,J)
      ENDIF
    END DO
  END DO
END DO
!
!  FNEW
!
FNEW = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 1,JM
  DO I = 2,IM
    DO K = 1,KB-1
      FNEW(I,J,K) = 0.5*(FIELD(I,J,K)+FIELD(I-1,J,K)) - ONESIX*(DIFF2(I,J,K)-DIFF2(I-1,J,K))
      FNEW(I,J,K) = FNEW(I,J,K)*DUM(I,J)
    END DO
  END DO
END DO
      

!$OMP PARALLEL DO PRIVATE(I,J,K,AREAT,AREATU)
DO J = 2,JM-1
  DO I = 2,IM
    IF (DUM(I,J).NE.0.) THEN
      DO K = 1,KB-1
      AREAT = DY*DUDZ(I,J,K)
      AREATU = AREAT*UADV(I,J,K)
      XFLUXD(I,J,K) = AH(I,J,K)*AREAT*(FIELD(I,J,K)-FIELD(I-1,J,K))/DX
      XFLUX(I,J,K) = AREATU*FNEW(I,J,K)
      XFLUX(I,J,K) = XFLUX(I,J,K) - XFLUXD(I,J,K)
      END DO
    ENDIF
  END DO
END DO
!
!  DIFF1
!
DIFF1 = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 2,JM
  DO I = 1,IM
    DO K = 1,KB-1
      DIFF1(I,J,K) = (FIELD(I,J,K) -FIELD(I,J-1,K))*DVM(I,J)
    END DO
  END DO
END DO
!
!  DIFF2
!
DIFF2 = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 1,JM-1
  DO I = 1,IM-1
    DO K = 1,KB-1
      SUM = DIFF1(I,J,K)+DIFF1(I,J+1,K)
      PROD = DIFF1(I,J,K)*DIFF1(I,J+1,K)
      IF (ABS(SUM).GT.EPSILON .AND. PROD.GT.EPSILON) THEN
        DIFF2(I,J,K) = 2.0*PROD/SUM
        DIFF2(I,J,K) = DIFF2(I,J,K)*FSM(I,J)
      ENDIF
    END DO
  END DO
END DO
!
!  FNEW
!
FNEW = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 2,JM
  DO I = 1,IM
    DO K = 1,KB-1
      FNEW(I,J,K) = 0.5*(FIELD(I,J,K)+FIELD(I,J-1,K)) - ONESIX*(DIFF2(I,J,K)-DIFF2(I,J-1,K))
      FNEW(I,J,K) = FNEW(I,J,K)*DVM(I,J)
    END DO
  END DO
END DO
!
!   Compute the sum of the advective and diffusive
!   fluxes  in Y-direction.
!
!$OMP PARALLEL DO PRIVATE(I,J,K,AREAT,AREATV)
DO J = 2,JM
  DO I = 2,IM-1
    IF (DVM(I,J).NE.0.) THEN
      DO K = 1,KB-1
      AREAT = DX*DVDZ(I,J,K)
      AREATV = AREAT*VADV(I,J,K)
      YFLUXD(I,J,K) = AREAT*AH(I,J,K)*((FIELD(I,J,K)-FIELD(I,J-1,K))/DY)
      YFLUX(I,J,K) = AREATV*FNEW(I,J,K)
      YFLUX(I,J,K) = YFLUX(I,J,K) - YFLUXD(I,J,K)
      END DO
    ENDIF
  END DO
END DO
!
!   Compute the advective fluxes in sigma-direction.
!
!
!  DIFF1
!
DIFF1 = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 1,JM
  DO I = 1,IM
    DO K = 2,KB
      DIFF1(I,J,K) = (FIELD(I,J,K-1) -FIELD(I,J,K))*FSM(I,J)
    END DO
  END DO
END DO
!
!  DIFF2
!
DIFF2 = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 1,JM
  DO I = 1,IM-1
    DO K = 1,KB-1
      SUM = DIFF1(I,J,K)+DIFF1(I,J,K+1)
      PROD = DIFF1(I,J,K)*DIFF1(I,J,K+1)
      IF (ABS(SUM).GT.EPSILON .AND. PROD.GT.EPSILON) THEN
        DIFF2(I,J,K) = 2.0*PROD/SUM
        DIFF2(I,J,K) = DIFF2(I,J,K)*FSM(I,J)
      ENDIF
    END DO
  END DO
END DO
!
!  FNEW
!
FNEW = 0.
!$OMP PARALLEL DO PRIVATE(I,J,K)
DO J = 2,JM
  DO I = 1,IM
    DO K = 2,KB-1
      FNEW(I,J,K) = 0.5*(FIELD(I,J,K)+FIELD(I,J,K-1)) - ONESIX*(DIFF2(I,J,K-1)-DIFF2(I,J,K))
      FNEW(I,J,K) = FNEW(I,J,K)*FSM(I,J)
    END DO
  END DO
END DO
AREA = DX*DY
!$OMP PARALLEL DO PRIVATE(I,J,K,AREATW)
DO J = 2,JM-1
  DO I = 2,IM-1
    IF(FSM(I,J).NE.0.) THEN
      DO K = 2,KB
        AREATW = AREA*W(I,J,K)
        ZFLUX(I,J,K) = AREATW*FNEW(I,J,K)
      END DO
    ENDIF
  END DO
END DO
!
!   Take a step to timestep n+1  by adding up the fluxes in and out of each cell.
!
!$OMP PARALLEL DO PRIVATE(I,J,K,NETFLUX,VOLUME,VOLUMEP)
DO J = 2,JM-1
  DO I =2,IM-1
    IF (FSM(I,J).GT.0.0) THEN
      DO K = 1,KB-1
        NETFLUX = (ZFLUX(I,J,K)-ZFLUX(I,J,K+1)) + &
   &            XFLUX(I+1,J,K) - XFLUX(I,J,K) + &
   &            YFLUX(I,J+1,K) - YFLUX(I,J,K)
        VOLUME = DX*DY*DZ(K)*(H(I,J)+ETA(I,J))
        VOLUMEP = DX*DY*DZ(K)*(H(I,J)+ETAP(I,J))
        FIELD(I,J,K) = (FIELDOLD(I,J,K)*VOLUMEP - DT*NETFLUX)/VOLUME
!
      END DO
    ENDIF
  END DO
END DO
!
  DELFIELD = FIELD - FIELDOLD
  FIELD = FIELD + FAVER
  FIELDOLD = FIELDOLD + FAVER
!
RETURN
END
