SUBROUTINE MODESPLIT(ISTEP,NDATE,IDX)
!
! MODESPLIT propagates the solution of the equations
!
! d(UD)/dt = -g*D*d(eta)/dx + f*V*D + d((KM/D)*(dU/dsigma))/dsigma +
!       d(AM*(d(U*D)/dy))/dy + d(AM*(d(U*D)/dx))/dx + deltaU 
!
! d(VD)/dt = -g*D*d(eta)/dy - f*U*D + d((KM/D)*(dV/dsigma))/dsigma +
!       d(AM*(d(V*D)/dx))/dx + d(AM*(d(V*D)/dy))/dy + deltaV 
!
! d(eta)/dt = -d(UA)/dx -d(VA)/dy
!
! with a time split method.
!
! The Depth integrated versions of these equations are
! propagated with a number of shorter timesteps (DTE).
! The Baroclinic remainders, UBCLIN and VBCLIN, are
! then propagated with a longer time step (DT). 
! U and V are then computed as the sums of the barotropic
! the baroclinic contributions.
!
! deltaU and deltaV are the sums of internal pressure terms and
! non-linear terms computed before the call to MODESPLIT
! by the routines INTERNAL and LAXWUV.
!
! The order in which the U and V equations are propagated
! is symmetric over a double step both in the barotropic
! steps and in the baroclinic steps.
!
! UA and VA are the depth integrated transports in
! x and y direction respectively.
!
!
USE MOD_BOUND
IMPLICIT NONE
INTEGER ISTEP,NDATE(5),IDX
!
! Local variables.
!
INTEGER I,J,K
REAL RX,RY,AREA
REAL, DIMENSION(IM,JM,KB) :: UV,VU,UBCLIN,VBCLIN,C3U,C3V
REAL, DIMENSION(IM,JM) :: AX,AY,UV2D,VU2D,UASUM,VASUM
REAL, DIMENSION(IM,JM) :: CORU,CORV
REAL, DIMENSION(IM,JM) :: C2U,C2V
REAL, DIMENSION(IM,JM) :: UAD,VAD,UASTAR,VASTAR,ETASTAR
REAL, DIMENSION(IM,JM) :: UAF,UAB,VAF,VAB,ETAB,ETAF
INTEGER IS
REAL DTE,FRAC
!
DTE = DT/N2D
RX = DTE/DX
RY = DTE/DY
!
!
!  Depth integrate the corrections DELTAU and DELTAV
!
AX = 0.
DO J = 1,JM
  DO I = 1,IM
    IF (DUM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        AX(I,J) = AX(I,J)+DUM(I,J)*DELTAU(I,J,K)*DZ(K)
      END DO
    ENDIF
  END DO
END DO
AY = 0.
DO J = 1,JM
  DO I = 1,IM
    IF (DVM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        AY(I,J) = AY(I,J)+DVM(I,J)*DELTAV(I,J,K)*DZ(K)
      END DO
    ENDIF 
  END DO
END DO
!
!
! Depth integrate U and V
!
UA = 0.
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      UA(I,J) = UA(I,J) + DUM(I,J)*UN(I,J,K)*DZ(K)
    END DO
  END DO
END DO
!
VA = 0.
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      VA(I,J) = VA(I,J) + DVM(I,J)*VN(I,J,K)*DZ(K)
    END DO
  END DO
END DO
!
!  Compute the baroclinic parts of U and V
!
UBCLIN = 0.
VBCLIN = 0.
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      UBCLIN(I,J,K) = UN(I,J,K) - UA(I,J)
      VBCLIN(I,J,K) = VN(I,J,K) - VA(I,J)
    END DO
  END DO
END DO
!
!  2-D loop N2D times
UASUM = 0.
VASUM = 0.
UAB = UA
VAB = VA
ETAB = ETA
UASTAR = UA
VASTAR = VA
ETASTAR = ETA
UAF = UA
VAF = VA
ETAF = ETA
!
DO IS = 1,N2D
!
!  If TIDE, then update the BCs every 2D time step.
!
  IF (TIDE) THEN
    CALL TIDEFRS(IS)
  ENDIF
!
!  River Specific
!
  UAD = 0.
  VAD = 0.
  IF (RIVLOG) THEN
    DO I = 1,NUMC
      IF (CODERIVER(I).EQ.1) THEN
        UAD(IRIVER(I),JRIVER(I)) = FRIVER(I)/DY
      ELSEIF (CODERIVER(I).EQ.2) THEN
        VAD(IRIVER(I),JRIVER(I)) = FRIVER(I)/DX
      ELSEIF (CODERIVER(I).EQ.3) THEN
        UAD(IRIVER(I)+1,JRIVER(I)) = -FRIVER(I)/DY
      ELSEIF (CODERIVER(I).EQ.4) THEN
        VAD(IRIVER(I),JRIVER(I)+1) = -FRIVER(I)/DX
      ELSE
        WRITE(*,*)'ERROR IN MODESPLIT-1'
        WRITE(*,*)I,CODERIVER(I)
        STOP
      ENDIF
    END DO
  ENDIF
!
! Compute contributions horizontal viscosity.
!
  IF (SMAG) THEN
    CALL SMAGOR2D
  ENDIF
  IF (VISCPOM) THEN
     CALL HORVISCUV2DPOM(C2U,C2V)
  ELSE
     CALL HORVISCUV2D(C2U,C2V)
  ENDIF
!
!
!  Add effects of wind stress and bottom friction and AX to C2U.
!
  DO J = 2,JM-1
    DO I = 2,IM
      C2U(I,J) = WUSURF(I,J)+WUBOT(I,J) + AX(I,J) + C2U(I,J)
    END DO
  END DO
!
! Add effects of wind stress and bottom friction and AY to C2V.
!
  DO J = 2,JM
     DO I = 2,IM-1
      C2V(I,J) = WVSURF(I,J)+WVBOT(I,J) + AY(I,J) + C2V(I,J)
     END DO
  END DO
!
!  Compute VA*D in U-points
!
  VU2D = 0.
!
! DO J = 2,JM-1
!   DO I = 2,IM
!     VU2D(I,J) = 0.25*(DV(I,J+1)*VA(I,J+1)+DV(I,J)*VA(I,J)+&
!          &  DV(I-1,J+1)*VA(I-1,J+1)+DV(I-1,J)*VA(I-1,J))
!   END DO
! END DO
!$omp parallel do
  DO J = 2,JM-1
     DO I = 2,IM
        IF (DUM(I,J).GT.0.) THEN
           IF (DVM(I,J+1).GT.0.) THEN
              VU2D(I,J) = VA(I,J+1)*DV(I,J+1)/HVSQR(I,J+1)
           ENDIF
           IF (DVM(I,J).GT.0.) THEN
              VU2D(I,J) = VU2D(I,J) + VA(I,J)*DV(I,J)/HVSQR(I,J)
           ENDIF
           IF (DVM(I-1,J+1).GT.0.) THEN
              VU2D(I,J) = VU2D(I,J) + VA(I-1,J+1)*DV(I-1,J+1)/HVSQR(I-1,J+1)
           ENDIF
           IF (DVM(I-1,J).GT.0.) THEN
              VU2D(I,J) = VU2D(I,J) + VA(I-1,J)*DV(I-1,J)/HVSQR(I-1,J)
           ENDIF
           VU2D(I,J) = 0.25*VU2D(I,J)*HUSQR(I,J)
        ENDIF
     END DO
  END DO

!
!Define U*D in V-points.
!
  UV2D = 0.
! DO J = 2,JM
!   DO I = 2,IM-1
!     UV2D(I,J) = 0.25*(DU(I+1,J)*UA(I+1,J)+DU(I,J)*UA(I,J)+&
!          & DU(I+1,J-1)*UA(I+1,J-1)+DU(I,J-1)*UA(I,J-1))
!   END DO
! END DO
!$omp parallel do
  DO J = 2,JM
     DO I = 2,IM-1
        IF (DVM(I,J).NE.0.) THEN
           IF (DUM(I+1,J).NE.0.) THEN
              UV2D(I,J) = UA(I+1,J)*DU(I+1,J)/HUSQR(I+1,J)
           ENDIF
           IF (DUM(I,J).NE.0.) THEN
              UV2D(I,J) = UV2D(I,J) + UA(I,J)*DU(I,J)/HUSQR(I,J)
           ENDIF
           IF (DUM(I+1,J-1).NE.0.) THEN
              UV2D(I,J) = UV2D(I,J) + UA(I+1,J-1)*DU(I+1,J-1)/HUSQR(I+1,J-1)
           ENDIF
           IF (DUM(I,J-1).NE.0.) THEN
              UV2D(I,J) = UV2D(I,J) + UA(I,J-1)*DU(I,J-1)/HUSQR(I,J-1)
           ENDIF
           UV2D(I,J) = 0.25*UV2D(I,J)*HVSQR(I,J)
        ENDIF
     END DO
  END DO
!
!  IF IS = 1, Compute solution at next time step with a predictor-corrector method similar to the trapzoidal method.
!
IF (IS.EQ.1) THEN
!
! Propagate the solution taking all terms into account.
!
  DO J = 2,JM-1
    DO I = 2,IM
      CORU(I,J) = COR(I,J)*VU2D(I,J) - GRAV*DU(I,J)* &
           &  (ETA(I,J)-ETA(I-1,J))/DX + C2U(I,J)
    END DO
  END DO
  DO J = 2,JM-1
    DO I = 2,IM
      IF (DUM(I,J).NE.0.) THEN
         UASTAR(I,J) = UA(I,J) + DTE*CORU(I,J)/DU(I,J)
      ENDIF
    END DO
  END DO
!
!  Then the V-equation to compute VASTAR.
!
!
  DO J = 2,JM
    DO I = 2,IM-1
      CORV(I,J) = -COR(I,J)*UV2D(I,J) - GRAV*DV(I,J)* &
           &  (ETA(I,J)-ETA(I,J-1))/DY +  C2V(I,J)
    END DO
  END DO
  DO J = 2,JM
    DO I = 2,IM-1
      IF (DVM(I,J).NE.0.) THEN
         VASTAR(I,J) = VA(I,J) + DTE*CORV(I,J)/DV(I,J)
      ENDIF
    END DO
  END DO
!
!
!
! Compute horizontal transports
!
  DO J = 2,JM-1
    DO I = 2,IM
      IF (DUM(I,J).NE.0.) THEN
        UAD(I,J) = UA(I,J)*DU(I,J)
      ENDIF
    END DO
  END DO
  DO J = 2,JM
    DO I = 2,IM-1
      IF (DVM(I,J).NE.0.) THEN
        VAD(I,J) = VA(I,J)*DV(I,J)
      ENDIF
    END DO
  END DO
!
!Then  the continuity eqn.
!
  DO J = 2,JM-1
    DO I = 2,IM-1
      ETASTAR(I,J) = ETA(I,J)-   &
     &  RX*(UAD(I+1,J)-UAD(I,J))- RY*(VAD(I,J+1)-VAD(I,J))
      ETASTAR(I,J) = ETASTAR(I,J)*FSM(I,J)
    END DO
  END DO
!
ELSE ! IF IS > 1
!
! Propagate the solution taking all terms into account.
! First a leapfrog predictor step
!
  DO J = 2,JM-1
    DO I = 2,IM
      CORU(I,J) = COR(I,J)*VU2D(I,J) - GRAV*DU(I,J)* &
           &  (ETA(I,J)-ETA(I-1,J))/DX + C2U(I,J)
    END DO
  END DO
  DO J = 2,JM-1
    DO I = 2,IM
      IF (DUM(I,J).NE.0.) THEN
         UASTAR(I,J) = UAB(I,J) + 2*DTE*CORU(I,J)/DU(I,J)
      ENDIF
    END DO
  END DO
!
!  Then the V-equation to compute VASTAR.
!
!
  DO J = 2,JM
    DO I = 2,IM-1
      CORV(I,J) = -COR(I,J)*UV2D(I,J) - GRAV*DV(I,J)* &
           &  (ETA(I,J)-ETA(I,J-1))/DY +  C2V(I,J)
    END DO
  END DO
  DO J = 2,JM
    DO I = 2,IM-1
      IF (DVM(I,J).NE.0.) THEN
         VASTAR(I,J) = VAB(I,J) + 2*DTE*CORV(I,J)/DV(I,J)
      ENDIF
    END DO
  END DO
!
!
!
! Compute horizontal transports
!
  DO J = 2,JM-1
    DO I = 2,IM
      IF (DUM(I,J).NE.0.) THEN
        UAD(I,J) = UA(I,J)*DU(I,J)
      ENDIF
    END DO
  END DO
  DO J = 2,JM
    DO I = 2,IM-1
      IF (DVM(I,J).NE.0.) THEN
        VAD(I,J) = VA(I,J)*DV(I,J)
      ENDIF
    END DO
  END DO
!
!Then  the continuity eqn.
!
  DO J = 2,JM-1
    DO I = 2,IM-1
      ETASTAR(I,J) = ETAB(I,J)-   &
     &  2*RX*(UAD(I+1,J)-UAD(I,J))- 2*RY*(VAD(I,J+1)-VAD(I,J))
      ETASTAR(I,J) = ETASTAR(I,J)*FSM(I,J)
    END DO
  END DO
ENDIF
!
!  Boundary conditions for the intermediate values of ETA, UA and VA
!
CALL BOUND2D(ETASTAR,UASTAR,VASTAR)
!
!
!  Then a corrector step with the theta- method
!
!
!  First in UA
!
!
!  Compute VA*D in U-points
!
  VU2D = 0.
! DO J = 2,JM-1
!   DO I = 2,IM
!     VU2D(I,J) = 0.25*(DV(I,J+1)*(THETA2D*VASTAR(I,J+1) + (1-THETA2D)*VA(I,J+1)) + &
!        & DV(I,J)*(THETA2D*VASTAR(I,J) + (1-THETA2D)*VA(I,J)) + &
!       &  DV(I-1,J+1)*(THETA2D*VASTAR(I-1,J+1) + (1-THETA2D)*VA(I-1,J+1)) + &
!       &  DV(I-1,J)*(THETA2D*VASTAR(I-1,J) + (1-THETA2D)*VA(I-1,J)))
!   END DO
! END DO
!
!$omp parallel do
  DO J = 2,JM-1
     DO I = 2,IM
        IF (DUM(I,J).GT.0.) THEN
           IF (DVM(I,J+1).GT.0.) THEN
              VU2D(I,J) = (THETA2D*VASTAR(I,J+1) + (1-THETA2D)*VA(I,J+1))*DV(I,J+1)/HVSQR(I,J+1)
           ENDIF
           IF (DVM(I,J).GT.0.) THEN
              VU2D(I,J) = VU2D(I,J) + (THETA2D*VASTAR(I,J) + (1-THETA2D)*VA(I,J))*DV(I,J)/HVSQR(I,J)
           ENDIF
           IF (DVM(I-1,J+1).GT.0.) THEN
              VU2D(I,J) = VU2D(I,J) + (THETA2D*VASTAR(I-1,J+1) + (1-THETA2D)*VA(I-1,J+1))*DV(I-1,J+1)/HVSQR(I-1,J+1)
           ENDIF
           IF (DVM(I-1,J).GT.0.) THEN
              VU2D(I,J) = VU2D(I,J) + (THETA2D*VASTAR(I-1,J) + (1-THETA2D)*VA(I-1,J))*DV(I-1,J)/HVSQR(I-1,J)
           ENDIF
           VU2D(I,J) = 0.25*VU2D(I,J)*HUSQR(I,J)
        ENDIF
     END DO
  END DO

!
! Propagate the solution taking all terms into account.
!
  DO J = 2,JM-1
    DO I = 2,IM
      CORU(I,J) = COR(I,J)*VU2D(I,J) - GRAV*DU(I,J)* &
           &  (THETA2D*(ETASTAR(I,J)-ETASTAR(I-1,J)) + &
           &   (1-THETA2D)*(ETA(I,J)-ETA(I-1,J)))/DX + C2U(I,J)
    END DO
  END DO
  DO J = 2,JM-1
    DO I = 2,IM
      IF (DUM(I,J).NE.0.) THEN
         UAF(I,J) = UA(I,J) + DTE*CORU(I,J)/DU(I,J)
      ENDIF
    END DO
  END DO
!
!
!  Then the VA-eqn.
!
  UV2D = 0.
! DO J = 2,JM
!   DO I = 2,IM-1
!     UV2D(I,J) = 0.25*(DU(I+1,J)*(THETA2D*UASTAR(I+1,J) + (1-THETA2D)*UA(I+1,J)) + &
!          & DU(I,J)*(THETA2D*UASTAR(I,J) + (1-THETA2D)*UA(I,J)) + &
!          & DU(I+1,J-1)*(THETA2D*UASTAR(I+1,J-1) + (1-THETA2D)*UA(I+1,J-1)) + &
!          & DU(I,J-1)*(THETA2D*UASTAR(I,J-1) + (1-THETA2D)*UA(I,J-1)))
!   END DO
! END DO
!
!$omp parallel do
  DO J = 2,JM
     DO I = 2,IM-1
        IF (DVM(I,J).NE.0.) THEN
           IF (DUM(I+1,J).NE.0.) THEN
              UV2D(I,J) = (THETA2D*UASTAR(I+1,J) + (1-THETA2D)*UA(I+1,J))*DU(I+1,J)/HUSQR(I+1,J)
           ENDIF
           IF (DUM(I,J).NE.0.) THEN
              UV2D(I,J) = UV2D(I,J) + (THETA2D*UASTAR(I,J) + (1-THETA2D)*UA(I,J))*DU(I,J)/HUSQR(I,J)
           ENDIF
           IF (DUM(I+1,J-1).NE.0.) THEN
              UV2D(I,J) = UV2D(I,J) + (THETA2D*UASTAR(I+1,J-1) + (1-THETA2D)*UA(I+1,J-1))*DU(I+1,J-1)/HUSQR(I+1,J-1)
           ENDIF
           IF (DUM(I,J-1).NE.0.) THEN
              UV2D(I,J) = UV2D(I,J) + (THETA2D*UASTAR(I,J-1) + (1-THETA2D)*UA(I,J-1))*DU(I,J-1)/HUSQR(I,J-1)
           ENDIF
           UV2D(I,J) = 0.25*UV2D(I,J)*HVSQR(I,J)
        ENDIF
     END DO
  END DO
!
!
  DO J = 2,JM
    DO I = 2,IM-1
      CORV(I,J) = -COR(I,J)*UV2D(I,J) - GRAV*DV(I,J)* &
           &  (THETA2D*(ETASTAR(I,J)-ETASTAR(I,J-1)) + &
           &   (1-THETA2D)*(ETA(I,J)-ETA(I,J-1)))/DY +  C2V(I,J)
    END DO
  END DO
  DO J = 2,JM
    DO I = 2,IM-1
      IF (DVM(I,J).NE.0.) THEN
         VAF(I,J) = VA(I,J) + DTE*CORV(I,J)/DV(I,J)
      ENDIF
    END DO
  END DO
!
!
!  Finally the ETA-eqn.
!
!
!
! Compute horizontal transports
!
  DO J = 2,JM-1
    DO I = 2,IM
      IF (DUM(I,J).NE.0.) THEN
        UAD(I,J) = (THETA2D*UASTAR(I,J) + (1-THETA2D)*UA(I,J))*DU(I,J)
      ENDIF
    END DO
  END DO
  DO J = 2,JM
    DO I = 2,IM-1
      IF (DVM(I,J).NE.0.) THEN
        VAD(I,J) = (THETA2D*VASTAR(I,J) + (1-THETA2D)*VA(I,J))*DV(I,J)
      ENDIF
    END DO
  END DO
!
!Then  the continuity eqn.
!
  DO J = 2,JM-1
    DO I = 2,IM-1
      ETAF(I,J) = ETA(I,J)-   &
     &  RX*(UAD(I+1,J)-UAD(I,J))- RY*(VAD(I,J+1)-VAD(I,J))
      ETAF(I,J) = ETAF(I,J)*FSM(I,J)
    END DO
  END DO
!
!  Boundary values for the final values at time step n+1
!
  CALL BOUND2D(ETAF,UAF,VAF)
!
  UASUM = UASUM + THETA2D*UASTAR + (1-THETA2D)*UA
  VASUM = VASUM + THETA2D*VASTAR + (1-THETA2D)*VA
  UAB = UA
  UA = UAF
  VAB = VA
  VA = VAF
  ETAB = ETA
  ETA = ETAF
!
!  End 2-D loop
!
END DO
!
UASUM = UASUM/N2D
VASUM = VASUM/N2D
!
!
! Compute the divergence free velocity fields.
!
UADV = 0.
VADV = 0.
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      UADV(I,J,K) = UBCLIN(I,J,K) + UASUM(I,J)
      VADV(I,J,K) = VBCLIN(I,J,K) + VASUM(I,J)
    END DO
  END DO
END DO
!
IF (SMAG) THEN
  CALL SMAGOR
ENDIF
IF (VISCPOM) THEN
  CALL HORVISCUVPOM(UBCLIN,VBCLIN,C3U,C3V)
ELSE 
  CALL HORVISCUV(UBCLIN,VBCLIN,C3U,C3V)
ENDIF
IF (MOD(ISTEP,2).EQ.1) THEN
   CALL STEPU3D(UBCLIN,VBCLIN,C3U,AX)
   CALL STEPV3D(UBCLIN,VBCLIN,C3V,AY)
ELSE
   CALL STEPV3D(UBCLIN,VBCLIN,C3V,AY)
   CALL STEPU3D(UBCLIN,VBCLIN,C3U,AX)
END IF
!
!  Include the effects of vertical viscosity on UBCLIN and VBCLIN
!
CALL VERTVISCUVB(UBCLIN,VBCLIN)
!  
!  After VERTVISCUVB the vertical integrals of UBCLIN and VBCLIN
!  are expected to be non-zero due to surface and bottom stress.
!  This is corrected for in the following loops.
!
DO J = 2,JM-1
  DO I = 2,IM
    IF (DUM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        UBCLIN(I,J,K) = UBCLIN(I,J,K) - DT*(WUBOT(I,J)+WUSURF(I,J))/DU(I,J)
        U(I,J,K) = UBCLIN(I,J,K) + UA(I,J)
      END DO
    ENDIF
  END DO
END DO
DO J = 2,JM
  DO I = 2,IM-1
    IF (DVM(I,J).NE.0.) THEN
      DO K = 1,KB-1
        VBCLIN(I,J,K) = VBCLIN(I,J,K) - DT*(WVBOT(I,J)+WVSURF(I,J))/DV(I,J)
        V(I,J,K) = VBCLIN(I,J,K) + VA(I,J)
      END DO
    ENDIF
  END DO
END DO
!
!   compute the sigma-coord. vertical velocity W
!   from the equation
! 
!   d(W)/d(sigma) = -(d(eta)/d(T) + d(U*D)/d(X) + d(V*D)/d(Y)) .
!
!
AREA = DX*DY
!
!   Compute the horisontal fluxes.
!
!   d(U*D)/d(X) times DX*DY
!
UV = 0.
VU = 0.
!
DO J = 1,JM
  DO I = 1,IM
    UV(I,J,1) = DZ(1)*UADV(I,J,1)
    DO K = 2,KB-1
      UV(I,J,K) = UV(I,J,K-1) + DZ(K)*UADV(I,J,K)
    END DO
  END DO
END DO
UV = UV*DY
!
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      UV(I,J,K) = UV(I,J,K)*DU(I,J)
    END DO
  END DO
END DO
!
!   d(V*D)/d(Y) times DX*DY
!
DO J = 1,JM
  DO I = 1,IM
    VU(I,J,1) = DZ(1)*VADV(I,J,1)
    DO K = 2,KB-1
      VU(I,J,K) = VU(I,J,K-1) + DZ(K)*VADV(I,J,K)
    END DO
  END DO
END DO
VU = VU*DX
DO K = 1,KB-1
  DO J = 1,JM
    DO I = 1,IM
      VU(I,J,K) = VU(I,J,K)*DV(I,J)
    END DO
  END DO
END DO
!
!  River spesific
!
IF (RIVLOG) THEN
  DO I = 1,NUMC
    IF (CODERIVER(I).EQ.1) THEN
      FRAC = DZ(1)/DZRIVER(I)
      UV(IRIVER(I),JRIVER(I),1) = FRAC*FRIVER(I)
      IF (KRIVER2(I).GT.1) THEN
        DO K = 2,KRIVER2(I)
          FRAC = DZ(K)/DZRIVER(I)
          UV(IRIVER(I),JRIVER(I),K) = UV(IRIVER(I),JRIVER(I),K-1) + &
     &                                   FRAC*FRIVER(I)
        END DO
      ENDIF
      IF (KRIVER2(I).LT.KB-1) THEN
        DO K = KRIVER2(I)+1,KB-1
          UV(IRIVER(I),JRIVER(I),K) = UV(IRIVER(I),JRIVER(I),KRIVER2(I))
        END DO
      ENDIF
    ELSEIF (CODERIVER(I).EQ.2) THEN
      FRAC = DZ(1)/DZRIVER(I)
      VU(IRIVER(I),JRIVER(I),1) = FRAC*FRIVER(I)
      IF (KRIVER2(I).GT.1) THEN
        DO K = 2,KRIVER2(I)
          FRAC = DZ(K)/DZRIVER(I)
          VU(IRIVER(I),JRIVER(I),K) = VU(IRIVER(I),JRIVER(I),K-1) + &
     &                                   FRAC*FRIVER(I)
        END DO
      ENDIF
      IF (KRIVER2(I).LT.KB-1) THEN
        DO K = KRIVER2(I)+1,KB-1
          VU(IRIVER(I),JRIVER(I),K) = VU(IRIVER(I),JRIVER(I),KRIVER2(I))
        END DO
      ENDIF
    ELSEIF (CODERIVER(I).EQ.3) THEN
      FRAC = DZ(1)/DZRIVER(I)
      UV(IRIVER(I)+1,JRIVER(I),1) = -FRAC*FRIVER(I)
      IF (KRIVER2(I).GT.1) THEN
        DO K = 2,KRIVER2(I)
          FRAC = DZ(K)/DZRIVER(I)
          UV(IRIVER(I)+1,JRIVER(I),K) = UV(IRIVER(I)+1,JRIVER(I),K-1) - &
     &                                   FRAC*FRIVER(I)
        END DO
      ENDIF
      IF (KRIVER2(I).LT.KB-1) THEN
        DO K = KRIVER2(I)+1,KB-1
          UV(IRIVER(I)+1,JRIVER(I),K) = UV(IRIVER(I)+1,JRIVER(I),KRIVER2(I))
        END DO
      ENDIF
    ELSEIF (CODERIVER(I).EQ.4) THEN
      FRAC = DZ(1)/DZRIVER(I)
      VU(IRIVER(I),JRIVER(I)+1,1) = -FRAC*FRIVER(I)
      IF (KRIVER2(I).GT.1) THEN
        DO K = 2,KRIVER2(I)
          FRAC = DZ(K)/DZRIVER(I)
          VU(IRIVER(I),JRIVER(I)+1,K) = VU(IRIVER(I),JRIVER(I)+1,K-1) - &
     &                                   FRAC*FRIVER(I)
        END DO
      ENDIF
      IF (KRIVER2(I).LT.KB-1) THEN
        DO K = KRIVER2(I)+1,KB-1
          VU(IRIVER(I),JRIVER(I)+1,K) = VU(IRIVER(I),JRIVER(I)+1,KRIVER2(I))
        END DO
      ENDIF
    ENDIF
  END DO
ENDIF
!WP = W
!
!   W = 0 for sigma = 0.
!
W(1:IM,1:JM,1) = 0.
!
!   Compute W from the equation
!   d(w)/d(sigma) = -(d(eta)/d(T) + d(U*D)/d(X) + d(V*D)/d(Y)) .
!
DO K = 1,KB-1
  DO J = 1,JM-1
    DO I = 1,IM-1
      W(I,J,K+1) = (UV(I+1,J,K) - UV(I,J,K) + &
     &             VU(I,J+1,K)-VU(I,J,K))/AREA + &
     &             ((-Z(K+1))*(ETA(I,J)-ETAP(I,J)))/DT
    END DO
  END DO
END DO
!
!  save corrections.
!
IF (IDX.EQ.1) THEN
  DELETAE = ETA - ETAP
  WE = W
  UADVE = UADV
  VADVE = VADV
!
!  version 4.0
!
  DELUN = U - UN
  U = UNM1 + 2.0*DELUN
  DELVN = V - VN
  V = VNM1 + 2.0*DELVN
ELSEIF (IDX.EQ.2) THEN
  DELETAI = ETA - ETAP
  WI = W
!
! version 4.0
!
  DELUSTAR = U - UN
  U = UN + (5.0/12.0)*DELUSTAR + (2.0/3.0)*DELUN - (1.0/12.0)*DELUNM1
  DELVSTAR = V - VN
  V = VN + (5.0/12.0)*DELVSTAR + (2.0/3.0)*DELVN - (1.0/12.0)*DELVNM1
!

ELSE
  WRITE(*,*)'ERROR IN MODETHETA',IDX
  STOP
ENDIF
!
RETURN
END
